@@ -126,7 +126,7 @@
 
 ;; Variables (not user-changeable)
 
-(defvar ess-version "5.2.6"
+(defvar ess-version "5.2.11"
   "Version of ESS currently loaded.")
 
 (defvar no-doc
@@ -625,49 +625,8 @@ Good for evaluating ESS code."
 
 ;;*;; Regular expressions
 
-;; FIXME : This is just for the S dialects;  need to define this for others,
-;; -----
-;;  {however  "XLS-mode" should just use standard lisp "beginning of function"}
-
-(defcustom ess-R-function-pattern
-  (concat
-   "\\(\\(" ; EITHER
-   "\\s\"" ; quote
-   "\\(\\sw\\|\\s_\\)+\\(<-\\)?" ; symbol (replacement?)
-   "\\s\"" ; quote
-   "\\)\\|\\(" ; OR
-   "\\(^\\|[ ]\\)" ; beginning of name
-   "\\(\\sw\\|\\s_\\)+" ; symbol
-   "\\)\\)" ; END EITHER OR
-   "\\s-*\\(<-\\|=\\)" ; whitespace, assign, whitespace/nl
-   "\\(\\(\\s-\\|\n\\)*\\s<.*\\s>\\)*" ; whitespace, comment
-   "\\(\\s-\\|\n\\)*function\\s-*(" ; whitespace, function keyword, parenthesis
-   )
-  "The regular expression for matching the beginning of an R function."
-  :group 'ess
-  :type 'regexp)
-
-(defcustom ess-S-function-pattern
-  ;; the same as "R" - but allowing "_" in assign
-  (concat
-   "\\(\\(" ; EITHER
-   "\\s\"" ; quote
-   "\\(\\sw\\|\\s_\\)+\\(<-\\)?" ; symbol (replacement?)
-   "\\s\"" ; quote
-   "\\)\\|\\(" ; OR
-;;   "\\<\\(\\sw\\|\\s_\\)+" ; symbol
-;;   "[0-9a-zA-Z0-9$.]+" ; symbol
-   "\\(^\\|[ ]\\)" ; beginning of name
-   "\\(\\sw\\|\\s_\\)+" ; symbol
-   "\\)\\)" ; END EITHER OR
-   "\\s-*\\(<-\\|_\\|=\\)" ; whitespace, assign, whitespace/nl
-   "\\(\\(\\s-\\|\n\\)*\\s<.*\\s>\\)*" ; whitespace, comment
-   "\\(\\s-\\|\n\\)*function\\s-*(" ; whitespace, function keyword, parenthesis
-   )
-  "The regular expression for matching the beginning of an S function."
-  :group 'ess
-  :type 'regexp)
-
+;; -- Note: Some variables not-to-customize moved to ./ess-mode.el :
+;; ess-set-function-start
 
 ;; Fixme: the following is just for S dialects :
 (defcustom ess-dumped-missing-re
@@ -737,21 +696,6 @@ Used in e.g., \\[ess-execute-objects] or \\[ess-display-help-on-object]."
   :group 'ess-command
   :type 'string)
 
-(defcustom ess-r-versions '( "R-1" "R-2")
-  "*List of partial strings for versions of R to access within ESS.
-Each string specifies the start of a filename.  If a filename
-beginning with one of these strings is found on `exec-path', a M-x
-command for that version of R is made available.  For example, if the
-file \"R-1.8.1\" is found and this variable includes the string
-\"R-1\", a function called `M-x R-1.8.1' will be available to run that
-version of R.
-If duplicate versions of the same program are found (which happens if
-the same path is listed on `exec-path' more than once), they are
-ignored by calling `ess-uniq-list'."
-  :group 'ess-R
-  :type '(repeat string))
-
-
 (defcustom ess-rterm-versions nil
 "*Construct ess-rterm-versions.  If you have versions of R in
 locations other than in ../../rw*/bin/Rterm.exe, relative to the
