@@ -92,6 +92,11 @@ An initialization file (dumped into the process) is specified by
 `inferior-ess-start-file', and `inferior-ess-start-args' is used to
 accompany the call for `inferior-ess-program'.
 
+When creating a new process, the process buffer replaces the
+current window if `inferior-ess-same-window' is non-nil.
+Alternatively, it can appear in its own frame if
+`inferior-ess-own-frame' is non-nil.
+
 \(Type \\[describe-mode] in the process buffer for a list of commands.)"
 
   ;; Use the current buffer if it is in inferior-ess-mode or ess-trans-mode
@@ -131,18 +136,14 @@ accompany the call for `inferior-ess-program'.
       ;; now the abomination:
       (ess-setq-vars-default ess-customize-alist)
 
+      ;; Emacs 22.0.50: ".. obsolete since 22.1;
+      ;;		use `comint-use-prompt-regexp' instead
       (setq-default comint-use-prompt-regexp-instead-of-fields nil); re set HACK!
       ;;>> Doesn't set ess-language,
       ;;>> => comint-input-sender is not set to 'ess-input-  ==> no input echo!
       ;;>> => that's why things fail:
       ;;>> (ess-setq-vars-local ess-customize-alist (current-buffer))
       ;;		 ======
-      ;;MM --- why set them here again when already "(let ..)" above ?
-      ;;   --> trying without (comment the following 4 lines):
-      ;; (setq temp-ess-dialect
-      ;;    (eval(cdr(assoc 'ess-dialect ess-customize-alist))))
-      ;; (setq temp-ess-lang
-      ;;    (eval(cdr(assoc 'ess-language ess-customize-alist))))
       )
 
     ;; run hooks now, to overwrite the above!
@@ -349,7 +350,7 @@ there is no process NAME)."
 	;; Add the process filter to catch certain output.
 	(set-process-filter (get-process proc-name)
 			    'inferior-ess-output-filter)
-	
+
 	(run-hooks 'ess-post-run-hook))
       (if (and inferior-ess-same-window (not inferior-ess-own-frame))
 	  (switch-to-buffer (process-buffer (get-process proc-name)))
@@ -1456,17 +1457,35 @@ to continue it."
    (format "(i-ess 3): curr-buf=%s, comint..echo=%s, comint..sender=%s,\n"
 	   (current-buffer) comint-process-echoes comint-input-sender))
 
-  ;; Completion support
+  ;;; Completion support ----------------
+
   ;; SJE: comint-dynamic-complete-functions is regarded as a hook, rather
-  ;; than a regular variable.
-  (add-hook 'comint-dynamic-complete-functions
-	    'ess-complete-object-name nil 'local)
-  (add-hook 'comint-dynamic-complete-functions
-	     'comint-replace-by-expanded-history 'append 'local)
+  ;; than a regular variable.  Note order of completion (thanks David Brahm):
+
   (add-hook 'comint-dynamic-complete-functions
 	    'ess-complete-filename 'append 'local)
+  (add-hook 'comint-dynamic-complete-functions
+ 	    'ess-complete-object-name 'append 'local)
+  (add-hook 'comint-dynamic-complete-functions
+	    'comint-replace-by-expanded-history 'append 'local)
 
+  ;; When a hook is buffer-local, the dummy function `t' is added to
+  ;; indicate that the functions in the global value of the hook
+  ;; should also be run.  SJE: I have removed this, as I think it
+  ;; interferes with our normal completion.
+  (remove-hook 'comint-dynamic-complete-functions 't 'local)
 
+  ;; MM: in *R* in GNU emacs and in Xemacs, the c*-dyn*-compl*-fun* are now
+  ;; (ess-complete-filename
+  ;;  ess-complete-object-name
+  ;;  comint-replace-by-expanded-history)
+
+  ;; However this fails in Xemacs 21.4.17 where the value in *shell* is
+  ;; -- the same as in GNU emacs *shell* :
+  ;; (comint-replace-by-expanded-history shell-dynamic-complete-environment-variable shell-dynamic-complete-command shell-replace-by-expanded-directory comint-dynamic-complete-filename)
+
+  ;; and the (Xemacs) global  'Default-value' is
+  ;; (comint-replace-by-expanded-history comint-dynamic-complete-filename)
 
   ;; (setq comint-completion-addsuffix nil) ; To avoid spaces after filenames
   ;; KH: next 2 lines solve.
@@ -1518,7 +1537,7 @@ to continue it."
 ;;  ___hack much improved by MM___ , both help(.) and ?... now work
 ;; FIXME: Note that  '??' nicely works in *R*, but
 ;;        'type ? topic' doesn't use ess-help {but display in *R*}
-(defconst inferior-R-1-input-help (format "help *(%s)" ess-help-arg-regexp))
+(defconst inferior-R-1-input-help (format "^ *help *(%s)" ess-help-arg-regexp))
 (defconst inferior-R-2-input-help (format "^ *\\? *%s" ess-help-arg-regexp))
 (defconst inferior-R-page         (format "^ *page *(%s)" ess-help-arg-regexp))
 
@@ -2096,8 +2115,8 @@ form completions."
  (ess-get-modtime-list))
 
 
-(defun ess-complete-filename nil
-  ;; Do file completion only within strings, or when the ! call is being used
+(defun ess-complete-filename ()
+  "Do file completion only within strings, or when ! call is being used."
   (if (comint-within-quotes
        (1- (process-mark (get-buffer-process (current-buffer)))) (point))
       ;; (- comint-last-input-start 1) (point))	 <- from S4 modeadds.
@@ -2109,7 +2128,11 @@ form completions."
       ;;    (looking-at "\\s-*!"))
       ;;  (comint-within-quotes comint-last-input-start (point)))
       (progn
-	(comint-replace-by-expanded-filename)
+	;;DBG (ess-write-to-dribble-buffer "ess-complete-f.name: within-quotes")
+	(if (featurep 'xemacs) ;; work around Xemacs bug
+	    (comint-dynamic-complete-filename)
+	  ;; GNU emacs and correctly working Xemacs:
+	  (comint-replace-by-expanded-filename))
 	;; always return t if in a string
 	t)))
 
