@@ -1,7 +1,7 @@
 ;;; ess-site.el --- user customization of ESS
 
 ;; Copyright (C) 1993 David M. Smith
-;; Copyright (C) 1997--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1997--2005 A.J. Rossini, Rich M. Heiberger, Martin
 ;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Original Author: David Smith <D.M.Smith@lancaster.ac.uk>
@@ -185,7 +185,7 @@ The ESS etc directory stores various auxillary files that are useful
 for ESS, such as icons.")
 
 (defvar ess-etc-directory-list
-  '("../etc/" "../etc/ess/" "../../etc/ess/" "./etc/")
+  '("../etc/ess/" "../etc/" "../../etc/ess/" "./etc/")
   "*List of directories, relative to `ess-lisp-directory', to search for etc.")
 
 (while (and (listp ess-etc-directory-list) (consp ess-etc-directory-list))
@@ -200,8 +200,8 @@ for ESS, such as icons.")
       (beep 0) (beep 0)
       (message (concat
 		"ERROR:ess-site.el:ess-etc-directory\n"
-		"Relative to ess-lisp-directory\n"
-		"At least one of ../etc, ../etc/ess, ../../etc/ess must exist!"))
+		"Relative to ess-lisp-directory, one of the following must exist:\n"
+		"../etc/ess, ../etc, ../../etc/ess or ./etc"))
       (sit-for 4))))
 
 (defvar ess-info-directory nil
@@ -211,15 +211,16 @@ The ESS info directory stores the ESS info files.")
 ;;(1.2) If ess.info is not found, then ess-lisp-directory/../doc/info is added
 ;; resurrecting Stephen's version with a bug-fix & xemacs compatibility
 (unless
+    ;; challenge: can this become more elegant?
     (member t
-	(mapcar 'file-exists-p
-	    (mapcar '(lambda (x) (concat (file-name-as-directory x) "ess.info"))
-		(if (featurep 'xemacs)
-		    Info-directory-list Info-default-directory-list))))
-    (add-to-list (if (featurep 'xemacs)
-		     'Info-directory-list 'Info-default-directory-list)
-		     (expand-file-name
-			 (concat ess-lisp-directory "/../doc/info/"))))
+	    (mapcar 'file-exists-p
+		    (mapcar
+		     '(lambda(x) (concat (file-name-as-directory x) "ess.info"))
+		     (if (featurep 'xemacs) Info-directory-list
+		       Info-default-directory-list))))
+  (add-to-list (if (featurep 'xemacs)
+		   'Info-directory-list 'Info-default-directory-list)
+	       (expand-file-name "../doc/info/" ess-lisp-directory)))
 
 ;;; (1.3) Files ending in .q and .S are considered to be S source files
 ;;; Files ending in .St are considered to be S transcript files
@@ -444,6 +445,7 @@ sending `inferior-ess-language-start' to S-Plus.")
 
 ;;; (1.7) Literate Data Analysis
 (require 'ess-noweb)
+(require 'ess-swv); for Sweave
 
 ;; ALWAYS:
 (require 'ess); -> loads ess-cust.el and more
@@ -521,6 +523,7 @@ sending `inferior-ess-language-start' to S-Plus.")
 (let ( (ess-sqpe-versions-created)
        (ess-rterm-versions-created)
        (ess-r-versions-created)
+       (ess-s-versions-created)
        (ess-versions-created)
        )
   (if ess-microsoft-p
@@ -530,8 +533,11 @@ sending `inferior-ess-language-start' to S-Plus.")
 	(setq ess-rterm-versions (ess-find-rterm))
 	(setq ess-rterm-versions-created
 	      (ess-rterm-versions-create))) ;; use ess-rterm-versions
-    (setq ess-r-versions-created
-	  (ess-r-versions-create)))	    ;; use ess-r-versions
+    (progn
+      (setq ess-s-versions-created
+	    (ess-s-versions-create))      ;; use ess-s-versions
+      (setq ess-r-versions-created
+	    (ess-r-versions-create))))	    ;; use ess-r-versions
 
   ;; Add the new defuns, if any, to the menu.
   ;; Check that each variable exists, before adding.
@@ -540,7 +546,9 @@ sending `inferior-ess-language-start' to S-Plus.")
 	(ess-flatten-list
 	 (mapcar (lambda(x) (if (boundp x) (symbol-value x) nil))
 		 '(ess-r-versions-created
-		   ess-rterm-versions-created ess-sqpe-versions-created))))
+		   ess-rterm-versions-created
+		   ess-s-versions-created
+		   ess-sqpe-versions-created))))
 
   (when ess-versions-created
     ;; new-menu will be a list of 3-vectors, of the form:
@@ -666,18 +674,18 @@ sending `inferior-ess-language-start' to S-Plus.")
 ;;;(setq ess-sas-edit-keys-toggle t)   ;; optional TAB and C-TAB in sas-mode
 ;;;   Use the function call (ess-sas-edit-keys-toggle)
 ;;;   to change the setting after the first SAS-mode buffer has been created.
-;;;   1c. You can also define C-TAB in all modes by specifying either
-;;;(setq ess-sas-global-unix-keys t) ;; optional C-TAB bound in all modes
-;;;(setq ess-sas-global-pc-keys t)   ;; optional C-TAB bound in all modes
-;;;	  See below.
+;;;   1c. You can also define C-TAB in all modes by Option 2b (below).
 ;;;
 ;;;   2. Managing submitted SAS jobs with function keys.
-;;;   2a. Default: Function keys retain their global bindings.
-;;;   2b. Options: Uncomment at most one of the following four lines.
-;;;(setq ess-sas-local-unix-keys t)  ;; [f2]--[f8] bound in sas-mode and related modes
-;;;(setq ess-sas-local-pc-keys t)    ;; [f2]--[f8] bound in sas-mode and related modes
-;;;(setq ess-sas-global-unix-keys t) ;; [f2]--[f8] bound in all modes
-;;;(setq ess-sas-global-pc-keys t)   ;; [f2]--[f8] bound in all modes
+;;;   2a. Default: To define the function keys in ESS[SAS] mode only,
+;;;   uncomment at most one of the following two lines.
+;;;(setq ess-sas-local-unix-keys t)  ;; F2-F12 bound in ESS[SAS] mode
+;;;(setq ess-sas-local-pc-keys t)    ;; F2-F12 bound in ESS[SAS] mode
+;;;
+;;;   2b. Options: To define the function keys in all modes,
+;;;   uncomment at most one of the following two lines.
+;;;(setq ess-sas-global-unix-keys t) ;; F2-F12 bound in all modes
+;;;(setq ess-sas-global-pc-keys t)   ;; F2-F12 bound in all modes
 ;;;
 ;;;   3. If it is more convenient to have "*Async Shell Command*"
 ;;;	 in same-window-buffer-names, then uncomment the following line
