@@ -19,19 +19,17 @@
 ;;; This code is experimental, and runs best on Emacs 21 and XEmacs
 ;;; 21.  It has been tested only on Linux machines.  All feedback
 ;;; appreciated.
-;;; To use this code, place the following in .emacs after you have
-;;; loaded ess-site:
 ;;;
-;;; (setq ess-use-toolbar t)
-;;; (if (fboundp 'tool-bar-mode) (tool-bar-mode t))	;Emacs 21.
-;;; (require 'ess-toolbar)
+;;; If your emacs can support images, the ESS toolbar should be loaded.
+;;; 
 ;;; If you see a toolbar, but no icons, check out the value of
 ;;; ess-icon-directory.
 ;;;
 ;;; The toolbar can be customized in several ways.  To see options, do:
 ;;; M-x customize-group RET ess-toolbar RET
-;;; If you change any of the variables, you may need to restart Emacs
-;;; to see any effect.
+;;; If you change any of the variables, you _may_ need to restart Emacs
+;;; to see any effect.  See also the documentation for ess-toolbar-items
+;;; if you wish to change its value.
 ;;;
 ;;; Technical issues.
 ;; Emacs vs XEmacs.
@@ -92,8 +90,18 @@ Each list element has three items:
 3. the tooltip doc string (XEmacs only; Emacs gets doc string from menu items.
 
 General toolbar items are also added to the ESS toolbar
-iff `ess-toolbar-own-icons' is nil."
+iff `ess-toolbar-own-icons' is nil.
+
+Setting this variable with setq doesn't take effect once you have
+loaded ess-site, unless you follow it by a call to
+`ess-make-toolbar' afterwards.  Instead, change its value using
+Custom, and then on all new ESS buffers you should see the
+toolbar has changed."
   :group 'ess-toolbar
+  :set (lambda (symbol value)
+	 (set-default symbol value)
+	 (if (fboundp 'ess-make-toolbar)
+	     (ess-make-toolbar)))
   :type '(repeat (list (function :tag "Function to run")
 		       (string  :tag "Icon")
 		       (string  :tag "Tooltip"))))
@@ -117,7 +125,11 @@ If `ess-icon-directory' is invalid, please report a bug.")
   "Make the ESS toolbar."
   (if (featurep 'xemacs)
       (ess-make-toolbar-xemacs)
-    (ess-make-toolbar-emacs)))
+    ;; Under Emacs, only worth building the toolbar if tool-bar-map is 
+    ;; available.  e.g. when running Emacs within a terminal, tool-bar-map
+    ;; is not available, so no need to make the tool-bar.
+    (if (boundp 'tool-bar-map)
+	(ess-make-toolbar-emacs))))
 
 (defun ess-make-toolbar-emacs ()
   "Make the ESS toolbar under Emacs."
@@ -145,7 +157,7 @@ If `ess-icon-directory' is invalid, please report a bug.")
     (expand-file-name (concat (cadr x) ".xpm") ess-icon-directory))
    (car x)				;function
    t
-   (caddr x)				;doc string
+   (nth 2 x)				;doc string
    ))
 
 (defvar ess-toolbar-xemacs-general
