@@ -1,17 +1,11 @@
 ;;; ess-trns.el --- Support for manipulating S transcript files
 
-;; Copyright (C) 1989-1994 Bates, Kademan, Ritter and Smith
-;; Copyright (C) 1997-2000 Richard M. Heiberger <rmh@fisher.stat.temple.edu>
-;;				Kurt Hornik <hornik@ci.tuwien.ac.at>
-;;				Martin Maechler <maechler@stat.math.ethz.ch>
-;;				A.J. (Tony) Rossini <rossini@stat.sc.edu>
+;; Copyright (C) 1989--1994 Bates, Kademan, Ritter and Smith
+;; Copyright (C) 1997--2005 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: David Smith <dsmith@stats.adelaide.edu.au>
-;; Maintainer: A.J. Rossini <rossini@stat.sc.edu>
-;; Created: 7 Jan 1994
-;; Modified: $Date: 2002/06/20 21:52:06 $
-;; Version: $Revision: 5.14 $
-;; RCS: $Id: ess-trns.el,v 5.14 2002/06/20 21:52:06 rsparapa Exp $
+;; Original Author: David Smith <dsmith@stats.adelaide.edu.au>
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
 
 ;; This file is part of ESS
 
@@ -104,6 +98,7 @@
   (define-key ess-transcript-mode-map "\C-c\C-d" 'ess-dump-object-into-edit-buffer)
   (define-key ess-transcript-mode-map "\C-c\C-t" 'ess-execute-in-tb)
   (define-key ess-transcript-mode-map "\C-c\t"	 'ess-complete-object-name)
+  (define-key ess-transcript-mode-map "\C-a"	 'comint-bol)
   (define-key ess-transcript-mode-map "\M-\t"	 'comint-replace-by-expanded-filename)
   (define-key ess-transcript-mode-map "\M-?"	 'comint-dynamic-list-completions)
   (define-key ess-transcript-mode-map "\C-c\C-k" 'ess-request-a-process)
@@ -127,41 +122,41 @@
  ess-transcript-mode-menu ess-transcript-mode-map
  "Menu for use in S transcript mode."
  '("ESS-trans"
-   ["What is this? (beta)" ess-mouse-me                  t]
-   ["Describe"	       describe-mode			 t]
-   ["About"	       (ess-goto-info "Transcript Mode") t]
-   ["Send bug report"  ess-submit-bug-report		 t]
+   ["What is this? (beta)" ess-mouse-me			t]
+   ["Describe"	       describe-mode			t]
+   ["About"	      (ess-goto-info "Transcript Mode") t]
+   ["Send bug report"  ess-submit-bug-report		t]
    "------"
-   ["Mark cmd group"   mark-paragraph	      t]
-   ["Previous prompt"  comint-previous-prompt t]
-   ["Next prompt"      comint-next-prompt     t]
+   ["Mark cmd group"   mark-paragraph		t]
+   ["Previous prompt"  comint-previous-prompt	t]
+   ["Next prompt"      comint-next-prompt	t]
    "------"
-   ["Send and move"  ess-transcript-send-command-and-move t]
-   ["Copy command"   ess-transcript-copy-command	  t]
-   ["Send command"   ess-transcript-send-command	  t]
-   ["Clean Region"   ess-transcript-DO-clean-region       t]))
+   ["Send and move" ess-transcript-send-command-and-move t]
+   ["Copy command"  ess-transcript-copy-command		t]
+   ["Send command"  ess-transcript-send-command		t]
+   ["Clean Region"  ess-transcript-DO-clean-region	t]
+   ["Switch S process" ess-switch-process		t]
+))
 
+(unless (featurep 'xemacs)
+  (if (featurep 'ess-trans)
+      (define-key ess-transcript-mode-map [menu-bar ess-trans]
+	(cons "ess-trans" ess-transcript-mode-menu))
+    (eval-after-load "ess-trans"
+      '(define-key ess-transcript-mode-map
+	 [menu-bar ess-trans]
+	 (cons "ess-trans"
+	       ess-transcript-mode-menu)))))
 
-
-(if (not (string-match "XEmacs" emacs-version))
-    (progn
-       (if (featurep 'ess-trans)
-	   (define-key ess-transcript-mode-map [menu-bar ess-trans]
-	     (cons "ess-trans" ess-transcript-mode-menu))
-	 (eval-after-load "ess-trans"
-			  '(define-key ess-transcript-mode-map
-			     [menu-bar ess-trans]
-			     (cons "ess-trans"
-				   ess-transcript-mode-menu))))))
-
-(defun ess-transcript-mode-xemacs-menu ()
+(when (featurep 'xemacs)
+  (defun ess-transcript-mode-xemacs-menu ()
   "Hook to install `ess-transcript-mode' menu for XEmacs (w/ easymenu)."
   (if 'ess-transcript-mode
 	(easy-menu-add ess-transcript-mode-menu)
     (easy-menu-remove ess-transcript-mode-menu)))
 
-(if (string-match "XEmacs" emacs-version)
-    (add-hook 'ess-transcript-mode-hook 'ess-transcript-mode-xemacs-menu))
+  (add-hook 'ess-transcript-mode-hook 'ess-transcript-mode-xemacs-menu))
+
 
 (defun ess-transcript-mode (alist &optional proc)
   "Major mode for manipulating {ESS} transcript files.
@@ -179,7 +174,7 @@ in the region, leaving only the S commands.  Other keybindings are:
   (require 'ess-inf)
   (kill-all-local-variables)
   (toggle-read-only t) ;; to protect the buffer.
-  (ess-setq-vars-local alist (current-buffer))
+  (ess-setq-vars-local alist); (current-buffer))
   (setq major-mode 'ess-transcript-mode)
   (setq mode-name "ESS Transcript")
   (use-local-map ess-transcript-mode-map)
@@ -205,7 +200,7 @@ in the region, leaving only the S commands.  Other keybindings are:
   ;; font-lock support
   (make-local-variable 'font-lock-defaults)
   (setq font-lock-defaults
-	'(ess-trans-font-lock-keywords nil nil ((?' . "."))))
+	'(inferior-ess-font-lock-keywords nil nil ((?' . "."))))
 
   ;;; Keep <tabs> out of the code.
   (make-local-variable 'indent-tabs-mode)
@@ -278,6 +273,12 @@ prompt from those lines that remain.  Prefix argument means to use
   (interactive "r")
   (ess-transcript-clean-region beg end 'In-ANY-case))
 
+(defun ess-transcript-clean-buffer ()
+  "Cleanup the whole buffer.
+Use point-min/max to obey narrow-to-region."
+  (interactive)
+  (ess-transcript-clean-region (point-min) (point-max)))
+
  ; Local variables section
 
 ;;; This file is automatically placed in Outline minor mode.
