@@ -1,6 +1,6 @@
 ;;; essa-sas.el -- clean-room implementation of many SAS-mode features
 
-;; Copyright (C) 1997--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1997--2005 A.J. Rossini, Rich M. Heiberger, Martin
 ;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Original Author: Rodney A. Sparapani <rsparapa@mcw.edu>
@@ -89,24 +89,22 @@ or `ess-sas-data-view-insight'."
 ;;'(("[pP][dD][fF]" . "/usr/local/bin/acroread") ("[eE]?[pP][sS]" . "/usr/local/bin/gv")))
     (let ((ess-tmp-alist nil)
         (ess-tmp-file nil))
+    
+    (setq ess-tmp-file (executable-find (if ess-microsoft-p "gsview32" "gsview")))
 
-    (setq ess-tmp-file (executable-find (if ess-microsoft-p "acrord32" "acroread")))
-
-    (if ess-tmp-file
-	(setq ess-tmp-alist (cons "[pP][dD][fF]" ess-tmp-file)))
-    (setq ess-tmp-file (executable-find "gv"))
+    (if (not ess-tmp-file) (setq ess-tmp-file (executable-find "gv")))
 
     (if (not ess-tmp-file) (setq ess-tmp-file (executable-find "ghostview")))
 
-    (if (not ess-tmp-file) (setq ess-tmp-file (executable-find
-	(if ess-microsoft-p "gsview32" "gsview"))))
-
     (if ess-tmp-file
-	(if ess-tmp-alist
-	    (setq ess-tmp-alist (list ess-tmp-alist
-		(cons "[eE]?[pP][sS]" ess-tmp-file)))
-	    (setq ess-tmp-alist (list (cons "[pP][dD][fF]" ess-tmp-file)
-		(cons "[eE]?[pP][sS]" ess-tmp-file))))))
+	(setq ess-tmp-alist (list (cons "[eE]?[pP][sS]" ess-tmp-file)
+	    (cons "[pP][dD][fF]" ess-tmp-file)))
+
+	(setq ess-tmp-file (executable-find (if ess-microsoft-p "acrord32" "acroread")))
+
+	(if ess-tmp-file
+	    (setq ess-tmp-alist (cons "[pP][dD][fF]" ess-tmp-file)))))
+
   "*Associate file name extensions with graphics image file viewers."
   :group 'ess-sas
   :type  'string)
@@ -116,8 +114,8 @@ or `ess-sas-data-view-insight'."
 ;;    :group 'ess-sas
 ;;)
 
-(defcustom ess-sas-log-max 2500000
-  "*If a .log file exceeds this many bytes, just \"refresh\" this many bytes."
+(defcustom ess-sas-log-max 0
+  "*If >0 and .log file exceeds this many bytes, just \"refresh\" this many bytes."
   :group 'ess-sas
   :type  'integer)
 
@@ -204,7 +202,7 @@ should set this variable to 'sh regardless of their local shell
   :group 'ess-sas
   :type  'string)
 
-(defcustom ess-sas-submit-pre-command
+(defcustom ess-sas-submit-pre-command ;;"nohup"
   (if (equal ess-sas-submit-method 'sh)
       ;; nice is tricky, higher numbers give you lower priorities
       ;; if you are using csh/tcsh, the default priority is 4
@@ -220,13 +218,17 @@ should set this variable to 'sh regardless of their local shell
       ;; this should avoid the necessity of each user needing to set this
       ;; variable correctly based on the shell that they use and provide
       ;; an environment where all shells are treated equally
-      (let* ((temp-shell (getenv "SHELL"))
-	     (temp-char (string-match "/" temp-shell)))
 
+      (let* ((temp-shell (getenv "SHELL"))
+	     ;; AJR: old CYGWIN versions return nil for (getenv
+	     ;; "SHELL"), so we need to deal with it 'cause I have to 
+	     (temp-char (if temp-shell
+			    (string-match "/" temp-shell)
+			  nil)))
 	(while temp-char
 	  (setq temp-shell (substring temp-shell (+ 1 temp-char)))
 	  (setq temp-char (string-match "/" temp-shell)))
-
+	
 	(cond ((or (equal temp-shell "csh") (equal temp-shell "tcsh"))
 	       "nohup nice +6")
 	      (t "nohup nice")))
@@ -262,15 +264,31 @@ to the shell on Windows when `ess-sas-submit-method' is 'sh."
   :group 'ess-sas
   :type  'number)
 
-(defvar ess-sas-tab-stop-alist
+(defcustom ess-sas-tab-stop-list
   '(4 8 12 16 20 24 28 32 36 40 44 48 52 56 60 64 68 72 76 80 84 88 92 96 100 104 108 112 116 120)
-  "List of tab stop positions used by `tab-to-tab-stop' in ESS[SAS].")
+  "List of tab stop positions used by `tab-to-tab-stop' in ESS[SAS]."
+  :group 'ess-sas)
 
-(defcustom ess-sas-temp-root "ess-temp"
-  "*The root of the temporary .sas file for `ess-sas-submit-region'."
+(defcustom ess-sas-temp-root "-temp"
+  "*Appended to root name of the temporary .sas file for `ess-sas-submit-region'."
   :group 'ess-sas
   :type  'string)
 
+(defvar ess-sas-versions '("sas")
+  "List of partial strings for versions of SAS to access within ESS.
+Each string specifies the start of a filename.  If a filename
+beginning with one of these strings is found on `exec-path', a M-x
+command for that version of SAS is made available.  For example, if the
+file \"sas8\" is found and this variable includes the string
+\"sas\", a function called `M-x SAS8' will be available to run that
+version of SAS.
+If duplicate versions of the same program are found (which happens if
+the same path is listed on `exec-path' more than once), they are
+ignored by calling `ess-uniq-list'.
+If you set this variable, you need to restart Emacs (and set this variable
+before ess-site is loaded) for it to take effect.")
+
+
 ;;; Section 2:  Function Definitions
 
 
@@ -306,7 +324,7 @@ to the shell on Windows when `ess-sas-submit-method' is 'sh."
     (interactive)
     (ess-sas-goto "log" 'revert)
     (goto-char (point-max))
-    (insert-file-contents (concat ess-sas-temp-root ".log"))
+    (insert-file-contents (concat (ess-sas-temp-root) ".log"))
     (save-buffer))
 
 (defun ess-sas-append-lst ()
@@ -314,7 +332,7 @@ to the shell on Windows when `ess-sas-submit-method' is 'sh."
     (interactive)
     (ess-sas-goto "lst" 'revert)
     (goto-char (point-max))
-    (insert-file-contents (concat ess-sas-temp-root ".lst"))
+    (insert-file-contents (concat (ess-sas-temp-root) ".lst"))
     (save-buffer))
 
 (defun ess-sas-backward-delete-tab ()
@@ -427,7 +445,7 @@ current buffer if nil."
  (save-excursion (let ((ess-tmp-sas-data nil)
     (ess-tmp-sas-data-view-fsview-statement ess-sas-data-view-fsview-statement)
     (ess-search-regexp
-    "[ \t=]\\([a-zA-Z_][a-zA-Z_0-9]*[.][a-zA-Z_][a-zA-Z_0-9]*\\)\\(&.*\\)?[. ,()\t;]")
+    "[ \t=]\\([a-zA-Z_][a-zA-Z_0-9]*[.][a-zA-Z_][a-zA-Z_0-9]*\\)\\(&.*\\)?[. ,()\t;/]")
     (ess-search-except
     "^\\([wW][oO][rR][kK]\\|[fF][iI][rR][sS][tT]\\|[lL][aA][sS][tT]\\)[.]"))
 
@@ -621,7 +639,7 @@ current buffer if nil."
 				  ess-temp-kermit-remote-directory))
 
 	      (if revert
-		  (if (and ess-sas-log-max (string-equal suffix "log")
+		  (if (and (> ess-sas-log-max 0) (string-equal suffix "log")
 			   (> (nth 7 (file-attributes ess-sas-temp-file))
 			      ess-sas-log-max))
 		      (progn
@@ -670,9 +688,11 @@ current buffer if nil."
     "\\|WARNING: Apparent symbolic reference .* not resolved."
     "\\|NOTE 485-185: Informat .* was not found or could not be loaded."
     "\\|WARNING: Length of character variable has already been set."
+    "\\|WARNING: Not all variables in the list "
+    "\\|WARNING: RUN statement ignored due to previous errors."
     "\\|Bus Error In Task\\|Segmentation Violation In Task"
     "\\|NOTE: Estimated G matrix is not positive definite."))
-	(ess-sas-save-point nil))
+	(ess-sas-save-point nil) (ess-sas-pop-mark nil))
 
   (if (ess-sas-goto "log" 'revert) (progn
 	(setq ess-sas-save-point (point))
@@ -810,17 +830,20 @@ optional argument is non-nil, then set-buffer rather than switch."
   (kill-buffer nil)
 )
 
-(eval-when-compile
+;(eval-when-compile
   (condition-case nil
       (progn
         (require 'rtf-support)
         (when (featurep 'rtf-support)
 
-(defun ess-sas-rtf-portrait ()
+(defun ess-sas-rtf-portrait (&optional ess-tmp-font-size)
 "Creates an MS RTF portrait file from the current buffer."
     (interactive)
     (ess-sas-file-path)
 
+    (if (equal ess-tmp-font-size nil) 
+	(setq ess-tmp-font-size "18"))
+
     (let
 	((ess-temp-rtf-file (replace-in-string ess-sas-file-path "[.][^.]*$" ".rtf")))
 	    ;(expand-file-name (buffer-name)) "[.][^.]*$" ".rtf")))
@@ -830,14 +853,14 @@ optional argument is non-nil, then set-buffer rather than switch."
 	(replace-regexp "\\\\fmodern .*;" "\\\\fmodern courier;" )
         (goto-char (point-min))
 
-        (while (replace-regexp "\\\\fs[0-9]+" "\\\\fs18" ) nil)
+        (while (replace-regexp "\\\\fs[0-9]+" (concat "\\\\fs" ess-tmp-font-size)) nil)
 
         (save-buffer)))
 
 (defun ess-sas-rtf-us-landscape ()
 "Creates an MS RTF US landscape file from the current buffer."
     (interactive)
-    (ess-sas-rtf-portrait)
+    (ess-sas-rtf-portrait "16")
     (ess-sas-goto "rtf" t)
     (goto-char (point-min))
     (forward-line 3)
@@ -849,7 +872,7 @@ optional argument is non-nil, then set-buffer rather than switch."
 (defun ess-sas-rtf-a4-landscape ()
 "Creates an MS RTF A4 landscape file from the current buffer."
     (interactive)
-    (ess-sas-rtf-portrait)
+    (ess-sas-rtf-portrait "16")
     (ess-sas-goto "rtf" t)
     (goto-char (point-min))
     (forward-line 3)
@@ -858,7 +881,7 @@ optional argument is non-nil, then set-buffer rather than switch."
 "\\landscape\\paperh11905\\paperw16837\\margl1800\\margr1800\\margt1440\\margb1440\\sectd\\sbknone\\lndscpsxn\\pgwsxn16837\\pghsxn11905\\marglsxn1800\\margrsxn1800\\margtsxn1440\\margbsxn1440\\ftnbj\\ftnstart1\\ftnrstcont\\ftnnar\\aenddoc\\aftnrstcont\\aftnstart1\\aftnnrlc\n"))
     (save-buffer))
 ))
-    (error nil)))
+    (error nil)) ;)
 
 (defun ess-sas-submit ()
   "Save the .sas file and submit to shell using a function that
@@ -866,7 +889,9 @@ depends on the value of  `ess-sas-submit-method'"
   (interactive)
   (ess-sas-file-path)
   (ess-sas-goto-sas)
-  (ess-save-and-set-local-variables)
+  (save-buffer)
+  (hack-local-variables)
+  ;(ess-save-and-set-local-variables)
 
   (cond
    ((eq ess-sas-submit-method 'apple-script)
@@ -927,7 +952,7 @@ should be ..."
     (interactive)
     (ess-sas-file-path)
     (write-region (region-beginning) (region-end)
-	(concat ess-sas-temp-root ".sas"))
+	(concat (ess-sas-temp-root) ".sas"))
 
     (save-excursion
       (ess-sas-goto-shell t)
@@ -944,7 +969,7 @@ should be ..."
     (comint-send-input)
 
     (insert (concat ess-sas-submit-pre-command " " ess-sas-submit-command
-          " " ess-sas-temp-root " " ess-sas-submit-post-command))
+          " " (ess-sas-temp-root) " " ess-sas-submit-post-command))
     (comint-send-input)
     )
 )
@@ -1017,6 +1042,10 @@ Keep in mind that the maximum command line length in MS-DOS is
   (setq left-margin (current-column))
 )
 
+(defun ess-sas-temp-root ()
+"Return `ess-sas-file-path' sans extension with `ess-sas-temp-root' appended."
+(concat (file-name-sans-extension ess-sas-file-path) ess-sas-temp-root))
+
 (defun ess-sas-transcript (&optional strip)
 "Comment .log messages to create a .sas program; use C-u to strip."
 (interactive "P")
@@ -1077,6 +1106,68 @@ Sometimes its necessary to wait for a shell prompt."
     )
   )
 
+(defun ess-sas-versions-create ()
+  "Generate the `M-x SASV' functions for starting other versions of SAS.
+See `ess-sas-versions' for strings that determine which functions are created.
+
+The local variable `ess-sas-versions-created' is used to return list of
+the new SAS defuns, if any, that were created.  The defuns will normally
+be placed on the menubar upon ESS initialisation."
+
+  ;; This works by creating a temp buffer where the template function is
+  ;; edited so that V is replaced by the version number
+  (let ((template "")
+	(beg)
+	(versions)
+	(version)
+	(eval-buf (get-buffer-create "*ess-temp-sas-evals*"))
+	(ess-sas-versions-created)
+	)
+    ;;
+    ;; This is the template function used for creating M-x SASV.
+    (setq template "(defun SASV (&optional start-args)
+  \"Call SASV, i.e., the SAS version 'SASV' using ESS.
+This function was generated by `ess-sas-versions-create'.\"
+  (interactive \"P\")
+  (let ((inferior-SAS-program-name \"SASV\"))
+    (SAS start-args)))
+
+")
+    (save-excursion
+      (set-buffer eval-buf)
+      ;; clear the buffer.
+      (delete-region (point-min) (point-max))
+
+      ;; Find which versions of SAS we want.  Remove the pathname, leaving just
+      ;; the name of the executable.
+      (setq versions
+	    (ess-uniq-list
+	     (mapcar 'file-name-nondirectory
+		     (apply 'nconc
+			    (mapcar 'ess-find-exec-completions
+				    ess-sas-versions)))))
+      (ess-write-to-dribble-buffer
+       (format "(SAS): ess-sas-versions-create making M-x defuns for %s"
+	       (mapconcat 'identity versions " ")))
+      (setq ess-sas-versions-created versions) ;keep copy for returning at end.
+      ;; Iterate over each string in VERSIONS, creating a new defun each time.
+      (while versions
+	(setq version (car versions)
+	      versions (cdr versions))
+	(setq beg (point))
+	(insert template)
+	(goto-char beg)
+	(while (search-forward "SASV" nil t)
+	  (replace-match version t t))
+	(goto-char (point-max))
+	)
+      ;; buffer has now been created with defuns, so eval them!
+      (eval-buffer)
+      (kill-buffer eval-buf)
+      )
+    ess-sas-versions-created))
+
+
 ;;; Section 3:  Key Definitions
 
 
